@@ -11,6 +11,7 @@ import json
 import requests
 import re
 from collections import OrderedDict
+from base64 import b64encode, b64decode
 
 from .draw import *
 
@@ -32,6 +33,8 @@ class Backend:
         self.config = config
         assert "server" in config
         assert "key" in config
+        assert "htpasswd_user" in config
+        assert "htpasswd_pwd" in config
 
     def get_api_url(self, path, use_prefix=True):
 
@@ -39,11 +42,13 @@ class Backend:
         server = self.config["server"]
 
         # Avoid double lines...
-        if server[-1] == "/" and path[0] == "/":
+        if server[-1] == "/":
+            server = server[:-1]
+        if ((len(path) > 0) and (path[0] == "/")):
             path = path[1:]
 
         if use_prefix:
-            apiversion = self.config.get("version", "v1.0")
+            apiversion = self.config.get("version", "v2")
             url = "{}/api/{}/{}".format(server, apiversion, path)
         else:
             url = "{}/{}".format(server, path)
@@ -63,9 +68,13 @@ class Backend:
         method = str(method).lower()
 
         key = self.config["key"]
+        user = self.config['htpasswd_user']
+        pwd = self.config['htpasswd_pwd']
+        auth = b64encode(f"{user}:{pwd}".encode('utf-8')).decode('utf-8')
+
         headers = {
-            "Authorization": "Token {}".format(key),
-            "Content-Type": "application/json",
+            "X-API-Key": key,
+            "Authorization": f"Basic {auth}",
             "Accept": "application/json",
         }
 
@@ -78,6 +87,7 @@ class Backend:
                     params=params,
                 )
             elif method == "post":
+                headers["Content-Type"] = "application/json"
                 response = requests.post(
                     url, verify=False, headers=headers, params=params, json=data
                 )
@@ -119,7 +129,7 @@ class Backend:
         return result
 
     def health(self):
-        url = self.get_api_url("system/health")
+        url = self.get_api_url("")
         result = self.call(url)
         return result
 
@@ -143,8 +153,8 @@ class Backend:
            pipeline (str): Name of the pipeline
 
         """
-        url = self.get_api_url("usecase/pipeline/runs")
-        result = self.call(url, {"usecase": usecase, "pipeline": pipeline})
+        url = self.get_api_url("dashboard/runs")
+        result = self.call(url, {"usecasename": usecase, "pipelinename": pipeline})
         return result
 
     def run_detail(self, usecase, pipeline, runid):
@@ -198,3 +208,29 @@ class Backend:
         url = self.get_api_url("usecase/task/run/detail")
         result = self.call(url, {"usecase": usecase, "task": task, "runid": runid})
         return result
+
+    def scheduler_jobs_list(self):
+        """
+        List scheduler jobs
+        """
+        url = self.get_api_url("/scheduler/api/v1/jobs", use_prefix=False)
+        result = self.call(url)
+        return result
+
+    def app_list(self):
+        """
+        List apps configured
+        """
+        url = self.get_api_url("dashboard/specs/")
+        result = self.call(url)
+        return result
+
+    def app_detail(self, name):
+        """
+        Show details of an app
+        """
+        url = self.get_api_url(f"dashboard/specs/detail/{name}")
+        result = self.call(url)
+        return result
+
+
