@@ -2,11 +2,7 @@ import re
 import traceback
 from texttable import Texttable
 
-
-def draw_table(backend, command, result, columns, extra={}):
-
-    # Now we have to beautify
-    data = result["data"]
+def draw_table(backend, command, data, columns, extra={}):
 
     t = Texttable(max_width=0)
     t.add_rows(
@@ -27,14 +23,11 @@ def draw_table(backend, command, result, columns, extra={}):
     print(t.draw())
 
 
-def draw(backend, command, result, columns, extra={}):
-    return draw_table(backend, command, result, columns, extra)
+def draw(backend, command, data, columns, extra={}):
+    return draw_table(backend, command, data, columns, extra)
 
 
-def draw_dict(backend, command, result, keys, extra={}):
-
-    # Now we have to beautify
-    data = result["data"]
+def draw_dict(backend, command, data, keys=None, extra={}):
 
     t = Texttable(max_width=0)
     t.add_rows(
@@ -46,19 +39,30 @@ def draw_dict(backend, command, result, keys, extra={}):
         ]
         + [[k, v] for k, v in extra.items()]
     )
-    print(t.draw())
+    t.draw()
 
     t = Texttable(max_width=0)
-    # t.set_deco(Texttable.HEADER)
-    t.add_rows([["Key", "Value"]] + [[c, data.get(c, "")] for c in keys])
+    if isinstance(data, dict):
+        if keys is None:
+            keys = data.keys()
+        t.set_deco(Texttable.HEADER)
+        t.add_rows([["Key", "Value"]] + [[c, data.get(c, "")] for c in keys])
+    elif isinstance(data, list):
+        if keys is None:
+            keys = data[0].keys()
+        #t.set_deco(Texttable.HEADER)
+        rows = [["Record", "Key", "Value"]]
+        for idx, d in enumerate(data):
+            rows.append([""]*3)
+            incrows = [[str(idx), c, d.get(c, "")] for c in keys]
+            rows.extend(incrows)
+        t.add_rows(rows)
 
     print(t.draw())
 
 
-def draw_run_detail(backend, command, result, extra={}):
+def draw_run_detail(backend, command, data, extra={}):
 
-    # Now we have to beautify
-    data = result["data"]
 
     t = Texttable(max_width=0)
     t.add_rows(
@@ -177,10 +181,7 @@ def draw_run_detail(backend, command, result, extra={}):
     print(t.draw())
 
 
-def draw_health(backend, command, result, extra={}):
-
-    # Now we have to beautify
-    data = result["data"]
+def draw_health(backend, command, data, extra={}):
 
     def fixedlength(paragraph, n):
         paragraph = paragraph.strip()
