@@ -10,6 +10,9 @@ from dateutil import parser as dateparser
 from datetime import datetime, timedelta
 from collections import defaultdict
 
+from google.cloud import logging_v2 as gcplogging
+from google.auth import load_credentials_from_file
+
 logger = logging.getLogger("app")
 
 #######################################
@@ -749,3 +752,70 @@ class S3DBTProcessor(ProcessorBase):
             db.insert("processed", log_group, log_stream, lastevent, filtered)
 
         logger.debug(f"Completed {log_group}")
+
+
+class StackDriverProcessor():
+    """
+    Read interface to GCP Stack driver. It has limitations in terms of
+    being able to read one GCP log stream alone (due to the use of
+    google.cloud.logging library instead of google service API.
+    """
+    def __init__(self, *args, **kwargs):
+        self.name = "Stackdriver"
+
+    def list_entries(self, spec, stream):
+        """
+        Args:
+          spec (object): Dict containing 'cred' (credentials path)
+          stream (object): Dict containing name (humanized name), logname (GCP log name e.g., projects/<project_id>/logs/<logName>), start
+
+        Returns:
+          iterator (object):  One entry at a time
+
+        Sample code::
+
+            spec = {
+                'cred': os.environ['CREDENTIALS_FILE']
+            }
+
+            stream = {
+                'name': "syslog",
+                "logname": "projects/scribble-internal-1/logs/syslog",
+                "start_time": "2022-05-25T07:50:00Z",
+                "end_time": "2022-05-25T08:50:00Z",
+            }
+
+            for entry in driver.list_entries(spec, stream):
+                print(entry)
+
+
+        """
+
+        cred = spec['cred']
+        sdclient = gcplogging.Client(credentials=cred)
+
+        name = stream['name']
+        logname = stream['logname']
+
+        sdlogger = sdclient.logger(name)
+
+        start_time = stream.get("start_time", "1970-01-01T00:00:00Z")
+        end_time = stream.get('end_time', datetime.utcnow().isoformat())
+
+        filtercmd = f"""\
+logName="{logname}"
+timestamp >= "{start_time}"
+timestamp <= "{end_time}"
+"""
+
+        if 'extra' in stream:
+            filtercmd += str(stream['extra'])
+
+        logger.debug(f"Accessing stackdriver: {name}",
+                     extra={
+                         'data': filtercmd
+                     })
+
+        for entry in sdlogger.list_entries(filter_=filtercmd):
+            yield entry
+
