@@ -1,6 +1,7 @@
 import os
 import io
 import re
+import json
 import numpy as np
 import pandas as pd
 import time
@@ -12,9 +13,10 @@ import matplotlib.pyplot as plt
 import logging
 
 from enrichsdk.contrib.lib.transforms import note
+from enrichsdk.contrib.lib.assets import profilespec
+from enrichsdk.datasets import TransformDoodle
 from enrichsdk.datasets import DynamicCustomDataset
-
-from .iofn import *
+from enrichsdk.utils import SafeEncoder
 
 logger = logging.getLogger("app")
 
@@ -95,91 +97,32 @@ class AnomaliesBase(Compute):
         '''
         return spec
 
-    def process_spec(self, spec, datasets):
+    def process_spec(self, spec, data):
         msg = ""
 
-        # check profile
-        if (not isinstance(spec, dict)) or (len(spec) == 0):
-            raise Exception("Spec should be a valid dict")
-        for p in ["name", "desc", "config"]:
-            if (p not in spec):
-                err = f"Invalid spec: {p} missing"
-                raise Exception(err)
-        for p in ["dataset", "detect"]:
-            if (p not in spec['config']):
-                err = f"Invalid spec->config: {p} missing"
-                raise Exception(err)
+        if data is None:
+            msg = "No dataset loaded" + "\n"
+            logger.exception(
+                f"Spec: {spec['name']} -- skipping",
+                extra={"transform": self.name}
+            )
+            return None
 
         name = spec["name"]
         config = spec["config"]
         handlers = self.get_handlers(spec)
 
-        # Name of the dataset to read
-        dataset = config["dataset"]
-
-        nature = None
-        datasetobj = None
-        if dataset in datasets:
-            # Clean match
-            datasetobj = datasets[dataset]
-            paths = datasetobj.paths
-            for path in paths:
-                if path['name'] == 'default':
-                    nature = path['nature']
-                    break
-            # Nothing has been found. So choose the first one
-            nature = paths[0]['nature']
-        else:
-            # Match dynamic datasets
-            for dname, dobj in datasets.items():
-                if not isinstance(dobj, DynamicCustomDataset):
-                    continue
-                if dataset.startswith(dobj.name):
-                    nature = dobj.paths[0]['nature']
-                    datasetobj = dobj
-                    break
-
-        if nature is None:
-            err = f"No nature found for dataset: {dataset}"
-            raise Exception(err)
-
-        try:
-            if isinstance(datasetobj, DynamicCustomDataset):
-                if nature == "s3":
-                    result = self.get_dataset_s3_dynamic(spec, datasetobj)
-            elif nature == "db":
-                result = self.read_db_source(spec)
-            elif nature == "s3":
-                result = self.get_dataset_s3(spec, datasetobj)
-            elif (
-                (generate is not None)
-                and (generate in handlers)
-                and (callable(handlers[generate]))
-            ):
-                result = handlers[generate](spec)
-            elif (generate is not None) and (hasattr(self, generate)):
-                result = getattr(self, generate)(spec)
-            else:
-                raise Exception(f"Invalid specification: {name}")
-        except:
-            logger.exception(
-                f"[{name}] generation failed", extra={"transform": self.name}
-            )
-            raise
-
-        # then, use the specified strategy to detect anomalies
+        # use the specified strategy to detect anomalies
         strategy = config.get("strategy")
         if strategy == None:
             # We are doing the default anomaly detection which is deviation from mean
-            data = self.process_spec_default(result, spec)
+            data = self.process_spec_default(data, spec)
         elif strategy == "custom":
             # Custom callback
-            data  = self.process_spec_custom(result, spec)
-        elif strategy not in valid_detection_strategies:
-            raise Exception(f"{strategy} is not a valid detection strategy")
+            data  = self.process_spec_custom(data, spec)
         else:
             # for now, we fallback to the default strategy
-            data  = self.process_spec_default(result, spec)
+            data  = self.process_spec_default(data, spec)
 
         # The spec processor can return multiple dataframes
         if isinstance(data, pd.DataFrame):
@@ -246,7 +189,7 @@ class AnomaliesBase(Compute):
 
         # Get hold of the data first...
         name = spec["name"]
-        orig_df = data[name]
+        orig_df = data
         total_samples = len(orig_df)
 
         metrics     = config.get("metrics", orig_df.columns)
@@ -255,7 +198,6 @@ class AnomaliesBase(Compute):
         dimensions  = config.get("dimensions", [])
         columns     = list(set(metrics + outputs + dimensions))
 
-
         msg += f"Combined set of columns: {columns}" + "\n"
         msg += f"{note(orig_df, 'Original DF')}" + "\n"
 
@@ -354,17 +296,17 @@ class AnomaliesBase(Compute):
 
         for group in groups:
             group_name  = group.get('group')
-            metrics     = group.get('metrics')
+            dimensions  = group.get('dimensions')
 
             # we don't have what we need, skip
-            if group_name == None or metrics == None:
+            if group_name == None or dimensions == None:
                 continue
 
             # make a copy of the df, we'll keep adding anomlay metrics to it
             df = orig_df[columns].copy()
 
-            points      = df[metrics].to_numpy()    # all data as an MxN matrix
-            centroid    = df[metrics].mean().values # the computed centroid of the dataset
+            points      = df[dimensions].to_numpy()    # all data as an MxN matrix
+            centroid    = df[dimensions].mean().values # the computed centroid of the dataset
             distances   = distance.cdist(points, np.array([centroid]), 'euclidean') # distances of each point to centroid
             distances   = np.reshape(distances, len(distances))
             stddev      = np.nanstd(points)         # std dev of distances
@@ -403,7 +345,7 @@ class AnomaliesBase(Compute):
             anomaly_stats[id] = {
                 "level": 'group',
                 "name": group_name,
-                "metrics": metrics,
+                "dimensions": dimensions,
                 "threshold": threshold,
                 "n_anomalies": n_anomalies,
                 "perc_anomalies": perc_anomalies,
@@ -422,7 +364,7 @@ class AnomaliesBase(Compute):
         #########
         # construct the DF for output
         #########
-        # concat for all metrics
+        # concat for all metrics+groups
         df = pd.concat(dfs)
         # reorder columns
         first_cols = ['id', 'level', 'name']
@@ -489,18 +431,16 @@ class AnomaliesBase(Compute):
             ],
         }
 
-        self.update_frame(source, df, lineage)
+        self.update_frame(source, f"Dataset: {dataset}", df, lineage)
 
         return df
 
-    def read_s3_data(self, filename, params, **kwargs):
-
+    def read_s3_data(self, filename, params={}):
         # assume we have a resolved s3fs object
         s3 = self.args['s3']
         if s3.exists(filename):
-            df = pd.read_csv(s3.open(filename),**params)
+            df = pd.read_csv(s3.open(filename), **params)
             return df
-
         return None
 
     def get_spec_sources(self, spec, datasets):
@@ -522,7 +462,7 @@ class AnomaliesBase(Compute):
 
         return {s: datasets[s] for s in sources if s in datasets}
 
-    def update_frame(self, source, df, lineage=None):
+    def update_frame(self, source, description, df, lineage=None):
 
         if isinstance(source, str):
             name = source
@@ -553,32 +493,28 @@ class AnomaliesBase(Compute):
 
         self.state.update_frame(name, detail)
 
-    def construct_dataset_list(self, anomalyspecs):
-        if not hasattr(self, 'get_dataset_registry'):
-            raise Exception(
-                "get_datasets expects get_dataset_registry method"
-            )
+    def load_dataset(self, spec):
+        name        = spec['name']
+        config      = spec['config']
+        source_id   = config['source_id']
+        dataset     = config['dataset']
 
-        # call the overloaded method to get the dataset registry
-        registry = self.get_dataset_registry()
+        datacred = self.args['datacred']
+        doodle = TransformDoodle(self, self.state, datacred)
 
-        # what are all the datasets in the spec
-        spec_datasets = []
-        for spec in anomalyspecs:
-            dataset = spec.get('config', {}).get('dataset')
-            if dataset == None:
-                continue
-            spec_datasets.append(dataset)
+        source, paths = doodle.get_source_paths(start=datetime.today() + timedelta(days=-7),
+                                                end=datetime.today(),
+                                                name=None,
+                                                source_id=source_id)
 
-        # iterate to keep only the datasets in the spec
-        datasets = {}
-        for dataset in registry.datasets:
-            name = dataset.name
-            for subset in dataset.subsets:
-                d = f"{name}-{subset['name']}"
-                datasets[d] = dataset
+        df = self.get_dataset_s3(spec, paths)
 
-        return datasets
+        msg = note(df, f"Input Dataset: {dataset}")
+        logger.debug(
+            f"Loaded dataset for spec: {name}", extra={"transform": self.name, "data": msg}
+        )
+
+        return df
 
     def get_window_dates(self, config, args):
         # get the window size
@@ -599,97 +535,55 @@ class AnomaliesBase(Compute):
         return window, start_date, end_date
 
 
-    def get_dataset_s3_dynamic(self, spec, datasetobj):
-        """
-        Use the dataset object to read the dataset
-        """
+    def get_dataset_s3(self, spec, paths):
+        '''
+        Gets all files from paths and puts them together
+        into a single dataframe. If self.args['cache']==True,
+        then this consolidated dataframe is cached / read from cache
+        as applicable.
+        '''
+        msg = ""
 
-        name = spec["name"]
-        config = spec["config"]
-        params = spec.get("params", {})
+        run_date    = self.args['run_date']
+        config      = spec['config']
+        dataset     = config['dataset']
 
-        # Look for URI and load it
-        uri = config.get('uri', config.get('filename'))
-        filename = uri.replace("s3://","")
-
-        # Load it
-        df = self.read_s3_data(filename, params)
-
-        logger.debug("Read {}".format(name), extra={"transform": self.name})
-
-        # Insert lineage if possible
-        lineage = {
-            "type": "lineage",
-            "transform": self.name,
-            "dependencies": [
-                {
-                    "type": "file",
-                    "nature": "input",
-                    "objects": [uri]
-                },
-            ],
-        }
-
-        self.update_frame(spec, df, lineage)
-
-        return {name: df}
-
-    def get_dataset_s3(self, spec, datasetobj):
-        """
-        Use the dataset object to read the dataset
-        """
-
-        if not hasattr(self, "get_dataset"):
-            raise Exception(
-                "get_dataset_s3 expects get_dataset method"
-            )
-
-        name = spec["name"]
-        config = spec["config"]
-        window, start_date, end_date = self.get_window_dates(config, self.args)
-
-        dataset = config["dataset"]
-        params = spec.get("params", {})
-        filename = config.get("filename")
-        if filename == None:
-            raise Exception(
-                "filename param in config not provided"
-            )
-
-        # => Whether to cache
         cache = self.args.get("cache", False)
-        cachename = f"{dataset}-{start_date}-{end_date}"
-        cachefile = f"cache/{self.name}-cache-" + cachename + ".csv"
+        cachename = f"{dataset}-{run_date}"
+        cachefile = f"cache/{self.name}-rawdata-cache-" + cachename + ".csv"
 
+        # read from cache if available
         if cache:
             try:
                 os.makedirs(os.path.dirname(cachefile))
             except:
                 pass
             if os.path.exists(cachefile):
-                logger.debug(
-                    "Read cached {}".format(name), extra={"transform": self.name}
-                )
-                df = pd.read_csv(cachefile, **params)
-                return {name: df}
+                msg = f"Location: {cachefile}" + "\n"
+                df = pd.read_csv(cachefile)
+                logger.debug(f"Read cached {dataset}", extra={"transform": self.name})
+                return df
 
-        df, metadata = datasetobj.read_data(
-            start_date,
-            end_date,
-            filename=filename,
-            readfunc=self.read_s3_data,
-            params=params,
-        )
+        # read from S3
+        dfs = []
+        for path in paths:
+            _df = self.read_s3_data(path)
+            if _df is None:
+                msg += f"Path not found, skipping: {path}" + "\n"
+                continue
+            msg += f"Read from path: {path}" + "\n"
+            dfs.append(_df)
+        df = pd.concat(dfs)
 
-        logger.debug("Read {}".format(name), extra={"transform": self.name})
+        logger.debug(f"Read fresh {dataset}", extra={"transform": self.name})
 
-        # Cache it for future use...
+        # Cache it for future use
         if cache:
             df.to_csv(cachefile, index=False)
 
         # Insert lineage if possible
         lineage = None
-        if ("files" in metadata) and (len(metadata["files"]) > 0):
+        if (len(paths) > 0):
             lineage = {
                 "type": "lineage",
                 "transform": self.name,
@@ -697,27 +591,20 @@ class AnomaliesBase(Compute):
                     {
                         "type": "file",
                         "nature": "input",
-                        "objects": [metadata["files"][-1]],
+                        "objects": [paths],
                     },
                 ],
             }
 
-        self.update_frame(spec, df, lineage)
+        if not self.state.has_frame(spec['name']):
+            self.update_frame(spec, f"Dataset: {dataset}", df, lineage)
 
-        return {name: df}
-
-    def get_identifier(self, spec):
-
-        identifier  = f"{spec['id']}-{spec['name']}"
-        identifier = re.sub(r'\W+', '_', identifier)
-        return identifier
+        return df
 
     def s3_store_result(self, spec, data):
-
-        id          = spec['id']
         name        = spec['name']
+        appname     = spec.get('app','common')
         namespace   = spec.get('namespace', 'default')
-        identifier  = self.get_identifier(spec)
         run_date    = self.args['run_date']
         s3          = self.args['s3']
         epoch       = time.time()
@@ -736,9 +623,14 @@ class AnomaliesBase(Compute):
         plotdata = data["plotdata"]
 
         # where are we storing it?
-        targetdir = os.path.join(self.args['s3root'], namespace, f"{identifier}/{run_date}/{epoch}")
-        anomaliesfile   = os.path.join(targetdir, "outliers.csv")
+        targetdir = os.path.join(self.args['s3root'], f"{appname}/{namespace}/{name}/{run_date}/{epoch}")
+
+        anomaliesfile   = os.path.join(targetdir, f"outliers.csv")
         statsfile       = os.path.join(targetdir, f"stats.json")
+        metadatafile    = os.path.join(targetdir, f"metadata.json")
+
+        metadata = self.get_default_metadata(self.state)
+        metadata['spec'] = spec
 
         ## plot the results
         msg = note(df, f"{name} - Anomalies Tagged") + "\n"
@@ -748,7 +640,7 @@ class AnomaliesBase(Compute):
         # for each level-name combination (indexed by id)
         for id in set(df['id'].values):
 
-            plotfile = os.path.join(targetdir, f"plot/{id}.png")
+            plotfile = os.path.join(targetdir, f"plot-{id}.png")
             msg += f"plot: {plotfile}" + "\n"
             data_df = plotdata[id]
 
@@ -764,8 +656,8 @@ class AnomaliesBase(Compute):
                 plot.set(title=f'Metric Distribution ({id})')
             else:
                 # for the groups level
-                metrics = stats['anomaly_stats'][id]['metrics']
-                data_df = data_df[metrics+['__is_anomaly__']]  # only retain the columns we need
+                dimensions = stats['anomaly_stats'][id]['dimensions']
+                data_df = data_df[dimensions+metrics+['__is_anomaly__']]  # only retain the columns we need
                 plot = sns.pairplot(data_df,
                                     hue='__is_anomaly__')
                 plot.fig.subplots_adjust(top=.95)
@@ -792,27 +684,42 @@ class AnomaliesBase(Compute):
                 fd.write(img_data.getbuffer())
 
         # write to s3
-        # anomalies
         with s3.open(anomaliesfile, 'w') as fd:
             df.to_csv(fd, index=False)
-
-        # stats
         with s3.open(statsfile, 'w') as fd:
-            json.dump(stats, fd)
+            json.dump(stats, fd, indent=4, cls=SafeEncoder)
+        with s3.open(metadatafile, 'w') as fd:
+            json.dump(metadata, fd, indent=4, cls=SafeEncoder)
 
         logger.debug("Wrote anomalies".format(name),
                         extra={"transform": self.name,
                                 "data": msg})
 
     def store_result(self, spec, data):
+        name    = spec['name']
+        config  = spec['config']
+        store   = config.get('store', {"sink": "s3"})
 
-        logger.debug("Using default store: s3",
-                     extra={
-                         'transform': self.name
-                     })
+        for f in ["sink"]:
+            if f not in store:
+                logger.exception(
+                    f"Store has no {f} param, skipping.",
+                    extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                )
+                return
 
-        # Now store in s3
-        return self.s3_store_result(spec, data)
+        sink = store['sink']
+        if sink == "s3":
+            # store in s3
+            self.s3_store_result(spec, data)
+        elif sink == "db":
+            # store in db
+            self.db_store_result(spec, data)
+        else:
+            logger.exception(f"Unknown store for dataset: {name}",
+                         extra={
+                             'transform': self.name
+                         })
 
     def process(self, state):
         """
@@ -826,21 +733,59 @@ class AnomaliesBase(Compute):
         self.state = state
 
         # Get the anomaly profile
-        profile = self.get_profile()
-
-        # Get specs from anomaly profile
-        specs = self.get_specs(profile)
-
-        # get the dataset lookup table
-        datasets = self.construct_dataset_list(specs)
+        is_valid, profile, msg = profilespec.get_profile(self, "policyapp.outliersv2")
+        if is_valid:
+            logger.debug(
+                f"Loaded profilespec",
+                extra={"transform": self.name, "data": msg}
+            )
+        else:
+            logger.exception(
+                f"Could not load profilespec",
+                extra={"transform": self.name, "data": msg}
+            )
+            raise Exception("could not load profilespec")
 
         # Now go through each spec and generate anomaly reports
-        for spec in specs:
-            enable = spec.get("enable", True)
-            if not enable:
+        for spec in profile:
+            ## first, some checks on the spec
+            do_process_spec = True
+            name = spec.get('name', 'NO_SPEC_NAME')
+
+            enabled = spec.get("active", True)
+            if not enabled:
+                logger.debug(
+                    f"Spec not enabled, skipping.",
+                    extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                )
+                do_process_spec = False
                 continue
 
-            # pre-process the spec
+            for f in ["name", "config"]:
+                if f not in spec:
+                    logger.exception(
+                        f"Spec has no {f} param, skipping.",
+                        extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                    )
+                    do_process_spec = False
+                    break
+            if do_process_spec == False:
+                continue
+
+            config = spec['config']
+
+            for f in ["source_id"]:
+                if f not in config:
+                    logger.exception(
+                        f"Spec config has no {f} param, skipping.",
+                        extra={"transform": self.name, "data": json.dumps(spec, indent=4)}
+                    )
+                    do_process_spec = False
+                    break
+            if do_process_spec == False:
+                continue
+
+            ## pre-process the spec
             spec = self.preprocess_spec(spec)
             logger.debug(f"Preproccessed spec: {spec['name']}",
                              extra={
@@ -848,17 +793,15 @@ class AnomaliesBase(Compute):
                                  'data': json.dumps(spec, indent=4)
                              })
 
-            # process the spec to detect outliers
-            try:
-                data = self.process_spec(spec, datasets)
+            ## we can now proceed with processing the spec
+            # frist, load the source data
+            data = self.load_dataset(spec)
 
-                # write the detected outliers
-                self.store_result(spec, data)
-            except:
-                logger.exception(f"Failed: {spec['name']}",
-                                 extra={
-                                     'transform': self.name
-                                 })
+            ## process the spec to detect outliers
+            data = self.process_spec(spec, data)
+
+            # write the detected outliers
+            self.store_result(spec, data)
 
         # Done
         logger.debug(
