@@ -8,6 +8,8 @@ import copy
 import shutil
 import logging
 import tempfile
+import traceback
+from importlib import import_module
 from inspect import signature
 import multiprocessing as mp
 import numpy as np
@@ -34,6 +36,7 @@ __all__ = [
     "FilesMixin",
     "CheckpointMixin",
     "EmailMixin",
+    "DatasetSpecMixin"
 ]
 
 
@@ -1329,3 +1332,39 @@ class EmailMixin(object):
                     "Unable to cleanup temporary directory",
                     extra={"transform": self.name, "data": tmpdir},
                 )
+
+class DatasetSpecMixin(object):
+    """
+    This is to be used with spec that identifies the
+    datasets to be loaded
+    """
+
+    def get_input_dataset(self, name):
+
+        tspec = self.transform_spec
+        if 'input_datasets' not in tspec['spec']:
+            raise Exception(f"input_datasets not present in the transform spec")
+
+        datasets = tspec['spec']['input_datasets']
+        dspec = None
+        for d in datasets:
+            if d['name'] == name:
+                dspec = d
+                break
+
+        if dspec is None:
+            raise Exception(f"Dataset not defined: {name}")
+
+        source = d.get('source', d.get('lib', None))
+        if source is None:
+            raise Exception(f"Unable to find dataset definition. Spec is missing 'lib' element")
+
+        try:
+            mod = import_module(source)
+            dataset = mod.get_dataset(name)
+        except:
+            # traceback.print_exc()
+            logger.exception(f"Unable to load: {source}")
+            return None
+
+        return dataset
