@@ -7,11 +7,13 @@ import traceback
 import pandas as pd
 from hashlib import sha256
 from s3fs import S3FileSystem
-from ..lib.exceptions import NoDataFound
 
+from ..lib.exceptions import NoDataFound
+from ..utils import SafeEncoder
 from dateutil import parser as dateparser
 from datetime import datetime, date, timedelta
 
+from .generators import handlers as shared_generators
 
 class DataSource(object):
     """
@@ -304,6 +306,37 @@ class DataSource(object):
 
         return re.search(pattern, filename) is not None
 
+    def get_match_spec(self):
+        """
+        Get a serializable specification of the matching
+        function.
+        """
+        return json.loads(json.dumps(self.match, cls=SafeEncoder))
+
+    def get_subset_match_spec(self, name, params={}):
+        """
+        Get a serializable specification of the matching
+        function for the subset of this dataset
+        """
+        spec = self.get_match_spec()
+
+        subset = None
+        for s in self.subsets:
+            subsetname = self.resolve_subset_name(name, params)
+            if name == subsetname:
+                subset = s
+                break
+
+        if subset is None:
+            raise Exception(f"Cannot find subset {name} in {self.name}")
+
+        if 'filename' in subset:
+            spec['filename'] = subset['filename']
+        elif 'pattern' in subset:
+            spec['filename'] = subset['pattern']
+
+        return spec
+
     def get_path(self, name, full=False, resolve=None):
         return self.get_path_by_name(name, full, resolve)
 
@@ -398,6 +431,7 @@ class Dataset(DataSource):
 
             {
                 "generate": "generate_datetime_daily",
+                "params": {},
                 "match": "match_datetime_pattern_range",
                 "pattern": "plpevents-%Y%m%d-%H%M%S",
             }
@@ -507,9 +541,9 @@ class Dataset(DataSource):
 
         generate = params["match"].get("generate", "generate_datetime_daily")
 
-        if ((isinstance(generate, str)) and (not hasattr(self, generate))) and (
-            not callable(generate)
-        ):
+        if ((isinstance(generate, str)) and
+            (not hasattr(self, generate)) and
+            (generate not in shared_generators)):
             raise Exception("Matching requirement {} missing/invalid".format(generate))
 
         compare = params["match"].get("compare", "compare_datetime_pattern_match_range")
@@ -549,10 +583,15 @@ class Dataset(DataSource):
 
         func = self.match["generate"]
         if isinstance(func, str):
-            func = getattr(self, func)
+            if func in shared_generators:
+                func = shared_generators[func]
+            elif hasattr(self, func):
+                func = getattr(self, func)
+            else:
+                raise Exception(f"Unable to find generator: {func}")
 
         # Generate all possible names
-        names = func(start, end)
+        names = func(self.match, start, end)
 
         # Now insert the full path...
         try:
@@ -566,33 +605,6 @@ class Dataset(DataSource):
         # Check if the name is in the names..
         return names
 
-    def generate_datetime_daily(self, start, end):
-        """
-        Given a start and an end, generate the datetime
-        objects corresponding to each run for each day.
-
-        Args:
-           start (datetime): Start of the time range
-           end (datetime): End of range. If None, default = start
-        """
-
-        pattern = self.match["pattern"]
-
-        names = []
-        current = start
-        while current <= end:
-            if isinstance(pattern, str):
-                name = current.strftime(pattern)
-            elif callable(pattern):
-                name = pattern(current)
-            else:
-                raise Exception("Unsupported pattern type")
-
-            names.append({"timestamp": current.isoformat(), "name": name})
-            current += timedelta(days=1)
-
-        return names
-
     def compare_datetime_pattern_match_range(self, name, start, end):
         """
         Given a start and an end, generate the datetime
