@@ -48,6 +48,15 @@ def load_customer_assets(context={}):
             if customer not in customer_roots:
                 customer_roots[customer] = os.path.join(customer_root, customer)
 
+    # Look at even the current working directory...
+    cwd = os.getcwd()
+    for parent in ["..", "../.."]:
+        root = os.path.abspath(os.path.join(cwd, parent))
+        customer = os.path.basename(root)
+        if customer in customer_roots:
+            continue
+        customer_roots[customer] = root
+
     # Now go through each of them...
     for customer, customer_dir in customer_roots.items():
 
@@ -62,7 +71,7 @@ def load_customer_assets(context={}):
 
         enrichfile = os.path.join(customer_dir, "enrich.json")
         if not os.path.exists(enrichfile):
-            logger.debug("Missing enrichfile. Skipping customer: {}".format(customer))
+            #logger.debug("Missing enrichfile. Skipping customer: {}".format(customer))
             continue
 
         try:
@@ -72,7 +81,7 @@ def load_customer_assets(context={}):
             continue
 
         if not enrich.get("enable", True):
-            logger.debug("Disabled customer. Skipping customer: {}".format(customer))
+            #logger.debug("Disabled customer. Skipping customer: {}".format(customer))
             continue
 
         for usecase in os.listdir(customer_dir):
