@@ -42,7 +42,7 @@ def attach_files(msg, attachments=[]):
         msg.attach(attach)
 
 
-def send_html_email(content, sender, receivers, subject, attachments=[]):
+def send_html_email(content, sender, receivers, subject, attachments=[], cred={}):
     """
     Send HTML email to target audience
     """
@@ -57,11 +57,11 @@ def send_html_email(content, sender, receivers, subject, attachments=[]):
     if isinstance(attachments, list) and len(attachments) > 0:
         attach_files(msg, attachments)
 
-    server = os.environ["EMAIL_HOST"]
-    username = os.environ["EMAIL_HOST_USER"]
-    password = os.environ["EMAIL_HOST_PASSWORD"]
-    port = os.environ.get("EMAIL_PORT", "587")
-    timeout = os.environ.get("EMAIL_TIMEOUT", "30")
+    server = cred.get("EMAIL_HOST", os.environ["EMAIL_HOST"])
+    username = cred.get("EMAIL_HOST_USER", os.environ["EMAIL_HOST_USER"])
+    password = cred.get("EMAIL_HOST_PASSWORD", os.environ["EMAIL_HOST_PASSWORD"])
+    port = cred.get("EMAIL_PORT", os.environ.get("EMAIL_PORT", "587"))
+    timeout = cred.get("EMAIL_TIMEOUT", os.environ.get("EMAIL_TIMEOUT", "30"))
 
     # Collect the body
     text = msg.as_string()
