@@ -14,6 +14,7 @@ from dateutil import parser as date_parser
 
 import pandas as pd
 
+from ..core.state import EnrichStateBase
 from ..core.frames import PandasDataFrame, DictDataFrame
 from ..lib import read_siteconf, read_versionmap
 from ..lib.context import Context
@@ -78,21 +79,15 @@ class MockRunManager(object):
         pass
 
 
-class MockState(object):
+class MockState(EnrichStateBase):
     def __init__(self, *args, **kwargs):
+
+        super().__init__(*args, **kwargs)
         self.name = "ExperimentalPipeline"
         self.runid = "exp-pipeline-20180922-192321"
-        self.config = kwargs.pop("config", None)
         self.state = {}
         self.notes = []
-        self.performance_notes = []
-        self.expectations = []
-        self.dataset_accesses = []
-        self.notifications = {"email": []}
         user = getpass.getuser()
-        self.pid = os.getpid()
-        self.start_time = datetime.now()
-        self.end_time = datetime.now()
         self.stats = {
             "platform": {
                 "node": platform.node(),
@@ -123,7 +118,7 @@ class MockState(object):
     def make_performance_note(
         self, name, description, condition, priority=1, message=""
     ):
-        self.performance_notes.append(
+        self.performance_summary.append(
             {
                 "name": name,
                 "description": description,
@@ -271,7 +266,7 @@ class MockConfig(object):
             context = context.asdict()
 
         self.name = "TestConf"
-        self.state = MockState(config=self)
+        self.display = self.name
         self.runid = "run-2015021"
         self.description = "Experimental pipeline for testing transforms"
         self.context = context
@@ -300,6 +295,8 @@ class MockConfig(object):
         ]:
             setattr(self, attr, self.data_root)
 
+        self.state = MockState(config=self)
+
     def get_usecase(self):
         return self.usecase.get("org", {}).get("name", "Unknown")
 
@@ -631,7 +628,7 @@ class MockConfig(object):
                 "name": self.name,
                 "runid": self.state.runid,
                 "notes": self.state.notes,
-                "performance": self.state.performance_notes,
+                "performance": self.state.performance_summary,
                 "stats": self.state.stats,
                 "start_time": self.state.start_time,
                 "end_time": self.state.end_time,
@@ -699,6 +696,10 @@ class MockConfig(object):
 
 
 class MockPipeline:
+    """
+    Simulate a pipeline
+    """
+
     def execute(self, transformcls, testdata, save_state=False, context={}):
 
         # => First create a mock pipeline.
