@@ -98,6 +98,7 @@ def process():
     Development:
        init:   Bootstrap modules including transforms*
        test:   Test transforms, manage datasets
+       doodle: Access Doodle metadata server
        manage: Manage services such as mongo
 
     \b
@@ -696,6 +697,87 @@ def _testdata_action(ctx, dataset, command):
 
 test.add_command(testdata)
 
+##########################################################
+# Doodle
+##########################################################
+@click.group("doodle", hidden=True)
+@click.option("--context", default=None, help="Environment file")
+@click.option("--cred", default="doodle", help="Doodle credential name")
+@click.pass_context
+def doodle(ctx, context, cred):
+    """
+    Access doodle server
+    """
+
+    # use the context file to set the environment
+    try:
+        context = lib.Context(context)
+    except Exception as e:
+        raise
+
+    context.set_env()
+    ctx.ensure_object(dict)
+    ctx.obj["context"] = context.asdict()
+    ctx.obj["cred"] = cred
+
+@doodle.command("list-catalogs")
+@click.pass_context
+def _list_soures(ctx):
+    """
+    List sources
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    sources = doodle.list_catalogs()
+    print(json.dumps(sources, indent=4))
+
+@doodle.command("show-source")
+@click.argument('source_id')
+@click.pass_context
+def _show_source(ctx, source_id):
+    """
+    Show source
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    source = doodle.get_source(source_id)
+    print(json.dumps(source, indent=4))
+    
+@doodle.command("list-sources")
+@click.pass_context
+def _list_soures(ctx):
+    """
+    List sources
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+    sources = doodle.list_sources()
+    print(json.dumps(sources, indent=4))
+
+@doodle.command("update-source")
+@click.argument('source_id')
+@click.argument('filename')
+@click.pass_context
+def _update_source_metadata(ctx, source_id, filename):
+    """
+    List sources
+    """
+
+    cred = enrichsdk.lib.get_credentials_by_name(ctx.obj['cred'])
+    doodle = datasets.Doodle(cred)
+
+    try:
+        update = json.load(open(filename))
+    except Exception as e:
+        print("Error!", e)
+        print("Unable to read filename. It should be a valid json file")
+
+    result = doodle.update_source(source_id, update)
+    print(json.dumps(result, indent=4))
+
 #########################################
 # API
 #########################################
@@ -1464,9 +1546,10 @@ process.add_command(_version)
 process.add_command(_start)
 process.add_command(init_)
 process.add_command(envsetup)
-process.add_command(api)
+#process.add_command(api)
 process.add_command(test)
-process.add_command(sample)
+process.add_command(doodle)
+#process.add_command(sample)
 process.add_command(manage)
 process.add_command(_show_log)
 
