@@ -47,7 +47,7 @@ def get_dates():
 
 
 @task
-def run_enrichcli(delay=0):
+def run_pipeline(delay=0):
     logger = prefect.context.get("logger")
     logger.debug("Starting: Run Pipeline")
 
@@ -67,7 +67,7 @@ def run_enrichcli(delay=0):
         # Add required parameters
         # "TransformName:Variable=Value",
         # "SortEvents:backend=hs",
-        "user=pingali",
+        "user=" + os.getlogin(),
     ]
 
     process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -77,11 +77,11 @@ def run_enrichcli(delay=0):
     logger.debug("Completed: Run pipeline")
 
 
-# https://crontab.guru/
-# UTC 2AM Everyday
-schedule = CronSchedule("0 2 */1 * *")
-with Flow("Running EnrichCLI", schedule=schedule) as flow:
-    data = run_enrichcli(delay=0)
+if __name__ == "__main__":
+    with Flow("Running Enrich Pipeline", schedule=None) as flow:
+        data = run_pipeline(delay=0)
+        # A way to force the dependencies
+        # data = run_another_pipeline(data)
 
-executor = LocalDaskExecutor(scheduler="processes")
-flow.run(executor=executor)
+    executor = LocalDaskExecutor(scheduler="processes")
+    flow.run(executor=executor)
